#
#  Name:    mxcontrol_func.tcl
#
#  Purpose: Tcl procedures used by the mxcontrol GUI.
#
#  Author:  William Lavender
#
#-----------------------------------------------------------------------
#
#  Copyright 2000 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

namespace eval MxControl {
	variable server_array
	variable num_servers
	variable config_filename
}

proc read_config_file { filename } {

	set MxControl::config_filename $filename

	if [ catch { open $filename "r" } config_file ] {

		set t "Unable to open config file '$filename'"

		tk_dialog .cant_open_file_dialog "File Open Error" \
				$t error 0 "OK"

		exit 1
	}

	set MxControl::num_servers 0

	set line [ gets $config_file ]

	while { [ eof $config_file ] == 0 } {

		incr MxControl::num_servers

		set temp_list [ concat $line ]

		set i $MxControl::num_servers

		set MxControl::server_array($i,host) \
			[ lindex $temp_list 0 ]

		set MxControl::server_array($i,port) \
			[ lindex $temp_list 1 ]

		set MxControl::server_array($i,logfile_name) \
			[ lindex $temp_list 2 ]

		set MxControl::server_array($i,start) \
			[ lindex $temp_list 3 ]

		set MxControl::server_array($i,stop) \
			[ lindex $temp_list 4 ]

		set MxControl::server_array($i,cleanup) \
			[ lindex $temp_list 5 ]

		set MxControl::server_array($i,logfile) ""
		set MxControl::server_array($i,status) "Unknown"
		set MxControl::server_array($i,server) ""
		set MxControl::server_array($i,old_logfile_size) 0

		set line [ gets $config_file ]
	}

	close $config_file
	
	return
}

proc create_main_window {} {

	set filename $MxControl::config_filename

	if { $MxControl::num_servers <= 0 } {

		set t "No servers were found in the config file '$filename'"

		tk_dialog .no_servers_dialog "No servers found" $t error 0 "OK"

		exit 1
	}

	# Create the frame containing the list of servers.

	set sf .server_frame

	frame $sf

	label $sf.name_title -text "Server"

	grid $sf.name_title -row 0 -column 0

	for { set i 1 } { $i <= $MxControl::num_servers } { incr i } {
		set stem $sf.server${i}

		set host $MxControl::server_array($i,host)
		set port $MxControl::server_array($i,port)
		set start $MxControl::server_array($i,start)
		set stop $MxControl::server_array($i,stop)
		set cleanup $MxControl::server_array($i,cleanup)
		set logfile_name $MxControl::server_array($i,logfile_name)

		set server_name "$host@$port"

		label ${stem}_name -text $server_name

		grid ${stem}_name -row $i -column 0 -sticky w

		label ${stem}_status -text "Unknown" -bg cyan

		grid ${stem}_status -row $i -column 1 -padx 5

		button ${stem}_start -text Start \
				-command [ code start_server $i ]

		if { $start == "" } {
			${stem}_start configure -state disabled
		}

		grid ${stem}_start -row $i -column 2

		button ${stem}_stop -text Stop \
				-command [ code stop_server $i ]

		if { $stop == "" } {
			${stem}_stop configure -state disabled
		}

		grid ${stem}_stop -row $i -column 3

		button ${stem}_view_log -text "View Log" \
				-command [ code view_log $i ]

		if { $logfile_name == "" } {
			${stem}_view_log configure -state disabled
		}

		grid ${stem}_view_log -row $i -column 4

		button ${stem}_cleanup -text "Cleanup" \
				-command [ code cleanup $i ]

		if { $cleanup == "" } {
			${stem}_cleanup configure -state disabled
		}

		grid ${stem}_cleanup -row $i -column 5
	}

	pack $sf

	button .dismiss -text Dismiss -command exit

	pack .dismiss
}

proc start_server { server_num } {

	set start_command $MxControl::server_array($server_num,start)

	update idletasks

	eval exec $start_command
}

proc stop_server { server_num } {

	set stop_command $MxControl::server_array($server_num,stop)

	update idletasks

	eval exec $stop_command
}

proc cleanup { server_num } {

	set cleanup_command $MxControl::server_array($server_num,cleanup)

	update idletasks

	eval exec $cleanup_command
}

proc view_log { i } {

	#
	# Create the log message window for this server.
	#

	set top .view_log_${i}

	set status [ catch { toplevel $top } ]

	#
	# If the window already exists, bring it to the top and return.
	#

	if { $status != 0 } {
		raise $top
		focus $top
		return
	}

	#
	# Try to open the log file.
	#

	set filename $MxControl::server_array($i,logfile_name)

	if { $filename == "" } {
		set t "No logfile is define for the server $host@$port"

		tk_dialog .no_logfile_dialog "No logfile define" \
			$t error 0 "OK"

		return
	}

	set status [ catch { open $filename "r" } file ]

	if { $status != 0 } {
		set t \
		"The log file '$filename' for server $host@$port cannot be read"

		tk_dialog .unreadable_logfile_dialog "Cannot read logfile" \
			$t error 0 "OK"

		return
	}

	set MxControl::server_array($i,logfile) $file

	#
	# Set the old logfile size to -1, so that show_new_messages will
	# be forced to try to read from the log file.
	#

	set MxControl::server_array($i,old_logfile_size) -1

	#
	# Set the title of the window.
	#

	set host $MxControl::server_array($i,host)
	set port $MxControl::server_array($i,port)

	wm title $top "MX log for server $host@$port"

	#
	# Put the name of the log file at the top.
	#

	label $top.filename -text "Log file:  $filename"

	pack $top.filename

	#
	# Create the message text window.
	#

	set window $top.log_window

	iwidgets::scrolledtext $window

	$window configure -visibleitems 80x24
	$window configure -hscrollmode dynamic
	$window configure -vscrollmode dynamic

	pack $window -expand yes -fill both

	button $top.dismiss -text Dismiss -command [ code destroy $top ]

	pack $top.dismiss

	#
	# Copy the text currently in the log file to the log message window.
	#

	after 1 show_new_messages $i $window
}

proc show_new_messages { i scrolled_text_window } {

	#
	# If the log window no longer exists, close the logfile and return
	# without rescheduling the event handler.
	#

	set status [ catch { $scrolled_text_window childsite } text_window ]

	if { $status != 0 } {
		close $MxControl::server_array($i,logfile)

		set MxControl::server_array($i,logfile) ""

		return
	}

	#
	# Is the log file bigger than it was the last time we checked?
	#

	set logfile_name $MxControl::server_array($i,logfile_name)

	set old_logfile_size $MxControl::server_array($i,old_logfile_size)

	set logfile_size [ file size $logfile_name ]

	#
	# If so, then try to read from the log file and copy what we have
	# received to the log window.
	#

	if { $logfile_size > $old_logfile_size } {

		set MxControl::server_array($i,old_logfile_size) $logfile_size

		set new_data [ read $MxControl::server_array($i,logfile) ]

		$text_window configure -state normal

		$scrolled_text_window insert end $new_data

		$text_window configure -state disabled

		$scrolled_text_window yview moveto 1.0
	}

	#
	# Schedule this event handler to be run again in 1 second.
	#

	after 1000 show_new_messages $i $scrolled_text_window
}

proc check_all_servers {} {
	set sf .server_frame

	for { set i 1 } { $i <= $MxControl::num_servers } { incr i } {
		set stem $sf.server${i}

		set old_server_status $MxControl::server_array($i,status)

		set server_status $old_server_status

		switch -- $server_status {
			Up {}
			default {
				connect_to_server $i
			}
		}

		set server_status $MxControl::server_array($i,status)

		switch -- $server_status {
			Unknown -
			Up {
				check_server_status $i
			}
		}

		set server_status $MxControl::server_array($i,status)

		if { $server_status != $old_server_status } {

			set start $MxControl::server_array($i,start)
			set stop $MxControl::server_array($i,stop)

			switch -- $server_status {
				Up {
					${stem}_status configure -text "Up"
					${stem}_status configure -bg green

					${stem}_start configure -state disabled

					if { $stop == "" } {
						${stem}_stop \
						    configure -state disabled
					} else {
						${stem}_stop \
						    configure -state normal
					}
				}
				Down {
					${stem}_status configure -text "Down"
					${stem}_status configure -bg red

					if { $start == "" } {
						${stem}_start \
						    configure -state disabled
					} else {
						${stem}_start \
						    configure -state normal
					}

					${stem}_stop configure -state disabled
				}
				default {
					${stem}_status configure -text \
					  "Unexpected status '$server_status'"
	
					${stem}_status configure -bg hotpink
				}
			}
		}
	}

	after 1000 check_all_servers
}

proc connect_to_server { i } {

	set host $MxControl::server_array($i,host)
	set port $MxControl::server_array($i,port)

	set status [ catch { MxNet::connect_to_server $host $port } server ]

	if { $status == 0 } {
		set MxControl::server_array($i,status) "Up"
		set MxControl::server_array($i,server) $server
	} else {
		set MxControl::server_array($i,status) "Down"
		set MxControl::server_array($i,server) ""
	}
}

proc check_server_status { i } {

	set server $MxControl::server_array($i,server)

	set field "mx_database.list_is_active"

	set status [ catch { MxNet::get $server $field } response_list ]

	if { $status == 0 } {
		set MxControl::server_array($i,status) "Up"
	} else {
		set MxControl::server_array($i,status) "Down"

		close $server

		set MxControl::server_array($i,server) ""
	}
}

